@@ -8,18 +8,19 @@ maps with pattern matching.
 Usage:
     chromosight detect  [--kernel-config=FILE] [--pattern=loops]
                         [--pearson=auto] [--win-size=auto] [--iterations=auto]
-                        [--win-fmt={json,npy}] [--force-norm] [--full]
+                        [--win-fmt={json,npy}] [--force-norm]
                         [--subsample=no] [--inter] [--tsvd] [--smooth-trend]
                         [--n-mads=5] [--min-dist=0] [--max-dist=auto]
                         [--no-plotting] [--min-separation=auto] [--dump=DIR]
-                        [--threads=1] [--perc-undetected=auto] <contact_map>
-                        [<output>]
+                        [--threads=1] [--perc-zero=auto]
+                        [--perc-undetected=auto] <contact_map> [<output>]
     chromosight generate-config [--preset loops] [--click contact_map]
                         [--force-norm] [--win-size=auto] [--n-mads=5]
                         [--threads=1] <prefix>
     chromosight quantify [--inter] [--pattern=loops] [--subsample=no]
                          [--win-fmt=json] [--kernel-config=FILE] [--force-norm]
-                         [--threads=1] [--full] [--n-mads=5] [--win-size=auto]
+                         [--threads=1] [--n-mads=5] [--win-size=auto] 
+                         [--perc-undetected=auto] [--perc-zero=auto]
                          [--no-plotting] [--tsvd] <bed2d> <contact_map> <output>
     chromosight test
 
@@ -44,82 +45,14 @@ Arguments for detect:
     contact_map                 The Hi-C contact map to detect patterns on, in
                                 bedgraph2d or cool format.
     output                      name of the output directory
-    -d, --dump=DIR              Directory where to save matrix dumps during
-                                processing and detection. Each dump is saved as
-                                a compressed npz of a sparse matrix and can be
-                                loaded using scipy.sparse.load_npz. Disabled
-                                by default.
-    -f, --full                  Enable 'full' convolution mode: The whole matrix
-                                is scanned all the way to edges and missing bins
-                                are masked. This will allow to detect very close
-                                to the diagonal and close to repeated sequences
-                                at the cost of memory and compute time.
-    -F, --force-norm            Re-compute matrix normalization (balancing) and
-                                overwrite weights present in the cool files instead
-                                of reusing them.
-    -I, --inter                 Enable to consider interchromosomal contacts.
-                                Warning: Experimental feature with very high
-                                memory consumption is very high, only use with
-                                small matrices.
-    -i, --iterations=auto       How many iterations to perform after the first
-                                template-based pass. Auto sets an appropriate
-                                value loaded from the kernel configuration
-                                file. [default: 1]
-    -k, --kernel-config=FILE    Optionally give a path to a custom JSON kernel
-                                config path. Use this to override pattern if
-                                you do not want to use one of the preset
-                                patterns.
-    -m, --min-dist=auto         Minimum distance from the diagonal (in base pairs).
-                                If this value is smaller than the kernel size, the
-                                kernel will be cropped to avoid overlapping the
-                                diagonal, up to a min.size of 7x7. [default: auto]
-    -M, --max-dist=auto         Maximum distance from the diagonal (in base pairs)
-                                at which pattern detection should operate. Auto
-                                sets a value based on the kernel configuration
-                                file and the signal to noise ratio. [default: auto]
-    -n, --no-plotting           Disable generation of pileup plots.
-    -N, --n-mads=5              Maximum number of median absolute deviations below
-                                the median of the logged bin sums distribution
-                                allowed to consider detectable bins. [default: 5]
-    -P, --pattern=loops         Which pattern to detect. This will use preset
-                                configurations for the given pattern. Possible
-                                values are: loops, borders, hairpins and
-                                centromeres. [default: loops]
-    -p, --pearson=auto          Pearson correlation threshold when assessing pattern
-                                probability in the contact map. A lesser value
-                                leads to potentially more detections, but more
-                                false positives. [default: auto]
-    -s, --subsample=INT         If greater than 1, subsample contacts from the
-                                matrix to INT contacts. If between 0 and 1, subsample
-                                a proportion of contacts instead. This is useful
-                                when comparing matrices with different
-                                coverages. [default: no]
-    -S, --min-separation=auto   Minimum distance required between patterns, in
-                                basepairs. If two patterns are closer than this
-                                distance in both axes, the one with the lowest
-                                score is discarded. [default: auto]
-    -t, --threads=1             Number of CPUs to use in parallel. [default: 1]
-    -T, --smooth-trend          Use isotonic regression to reduce noise at long
-                                ranges caused by detrending. Do not enable this
-                                for circular genomes.
-    -u, --perc-undetected=auto  Maximum percentage of empty pixels in windows
-                                allowed to keep detected patterns. [default: auto]
-    -w, --win-fmt={json,npy}    File format used to store individual windows
-                                around each pattern. Window order match
-                                patterns inside the associated text file.
-                                Possible formats are json and npy. [default: json]
-    -W, --win-size=auto         Window size (width), in pixels, to use for the
-                                kernel when computing correlations. The pattern
-                                kernel will be resized to match this size. If
-                                the pattern must be enlarged, linear
-                                interpolation is used to fill between pixels.
-                                If not specified, the default kernel size will
-                                be used instead. [default: auto]
-    -V, --tsvd                  Enable kernel factorisation via truncated svd.
-                                This should accelerate detection in most cases,
-                                at the cost of slight inaccuracies. The singular
-                                matrices are truncated so that 99.9% of the
-                                information contained in the kernel is retained.
+
+Arguments for quantify:
+    bed2d                       Tab-separated text files with columns chrom1, start1
+                                end1, chrom2, start2, end2. Each line correspond to
+                                a pair of positions (i.e. a position in the matrix).
+    contact_map                 Path to the contact map, in bedgraph2d or
+                                cool format.
+    output                      output directory where files should be generated.
 
 Arguments for generate-config:
     prefix                      Path prefix for config files. If prefix is a/b,
@@ -133,13 +66,71 @@ Arguments for generate-config:
                                 user to build the kernel. Warning: memory-heavy,
                                 reserve for small genomes or subsetted matrices.
 
-Arguments for quantify:
-    bed2d                       Tab-separated text files with columns chrom1, start1
-                                end1, chrom2, start2, end2. Each line correspond to
-                                a pair of positions (i.e. a position in the matrix).
-    contact_map                 Path to the contact map, in bedgraph2d or
-                                cool format.
-    output                      output directory where files should be generated.
+Basic options: 
+    -F, --force-norm            Re-compute matrix normalization (balancing) and
+                                overwrite weights present in the cool files instead
+                                of reusing them.
+    -I, --inter                 Enable to consider interchromosomal contacts.
+                                Warning: Experimental feature with high memory
+                                consumption, only use with small matrices.
+    -m, --min-dist=auto         Minimum distance from the diagonal (in base pairs).
+                                at which detection should operate. [default: auto]
+    -M, --max-dist=auto         Maximum distance from the diagonal (in base pairs) 
+                                for detection. [default: auto]
+    -P, --pattern=loops         Which pattern to detect. This will use preset
+                                configurations for the given pattern. Possible
+                                values are: loops, borders, hairpins and
+                                centromeres. [default: loops]
+    -p, --pearson=auto          Pearson correlation threshold when detecting patterns
+                                in the contact map. Lower values leads to potentially
+                                more detections, but more false positives. [default: auto]
+    -s, --subsample=INT         If greater than 1, subsample INT contacts from the
+                                matrix. If between 0 and 1, subsample a proportion of
+                                contacts instead. Useful when comparing matrices with
+                                different coverages. [default: no]
+    -t, --threads=1             Number of CPUs to use in parallel. [default: 1]
+    -u, --perc-undetected=auto  Maximum percentage of non-detectable pixels (nan) in
+                                windows allowed to report patterns. [default: auto]
+    -z, --perc-zero=auto        Maximum percentage of empty (0) pixels in windows
+                                allowed to report patterns. [default: auto]
+
+Advanced options:
+    -d, --dump=DIR              Directory where to save matrix dumps during
+                                processing and detection. Each dump is saved as
+                                a compressed npz of a sparse matrix and can be
+                                loaded using scipy.sparse.load_npz. 
+    -i, --iterations=auto       How many iterations to perform after the first
+                                template-based pass. [default: 1]
+    -k, --kernel-config=FILE    Optionally give a path to a custom JSON kernel
+                                config path. Use this to override pattern if
+                                you do not want to use one of the preset
+                                patterns.
+    -n, --no-plotting           Disable generation of pileup plots.
+    -N, --n-mads=5              Maximum number of median absolute deviations below
+                                the median of the bin sums distribution allowed to
+                                consider detectable bins. [default: 5]
+    -S, --min-separation=auto   Minimum distance required between patterns, in
+                                basepairs. If two patterns are closer than this
+                                distance in both axes, the one with the lowest
+                                score is discarded. [default: auto]
+    -T, --smooth-trend          Use isotonic regression when detrending to reduce
+                                noise at long ranges. Do not enable this for circular
+                                genomes.
+    -V, --tsvd                  Enable kernel factorisation via truncated svd.
+                                Accelerates detection, at the cost of slight
+                                inaccuracies. Singular matrices are truncated to
+                                retain 99.9% of the information in the kernel.
+    -w, --win-fmt={json,npy}    File format used to store individual windows
+                                around each pattern. Window order matches
+                                patterns inside the associated text file.
+                                Possible formats are json and npy. [default: json]
+    -W, --win-size=auto         Window size (width), in pixels, to use for the
+                                kernel when computing correlations. The pattern
+                                kernel will be resized to match this size. Linear 
+                                linear interpolation is used to fill between pixels.
+                                If not specified, the default kernel size will
+                                be used instead. [default: auto]
+
 
 """
 import numpy as np
@@ -174,11 +165,11 @@ pearson set to 0.3 based on config file.
 max_dist set to 2000000 based on config file.
 min_dist set to 20000 based on config file.
 min_separation set to 5000 based on config file.
-max_perc_undetected set to 10.0 based on config file.
+max_perc_zero set to 30.0 based on config file.
 Matrix already balanced, reusing weights
 Preprocessing sub-matrices...
 Detecting patterns...
-21 patterns detected
+50 patterns detected
 """
 
 
@@ -212,12 +203,13 @@ def _override_kernel_config(param_name, param_value, param_type, config):
 def cmd_quantify(args):
     bed2d_path = args["<bed2d>"]
     mat_path = args["<contact_map>"]
-    full = args["--full"]
     output = pathlib.Path(args["<output>"])
     n_mads = float(args["--n-mads"])
     pattern = args["--pattern"]
     inter = args["--inter"]
     kernel_config_path = args["--kernel-config"]
+    perc_zero = args["--perc-zero"]
+    perc_undetected = args["--perc-undetected"]
     plotting_enabled = False if args["--no-plotting"] else True
     threads = int(args["--threads"])
     force_norm = args["--force-norm"]
@@ -262,7 +254,8 @@ def cmd_quantify(args):
     max_diag = hic_genome.clr.shape[0] * hic_genome.clr.binsize
     cfg["max_dist"] = min(furthest, max_diag)
     cfg["min_dist"] = 0
-    cfg["max_perc_undetected"] = 100
+    cfg = _override_kernel_config('max_perc_zero', perc_zero, float, cfg)
+    cfg = _override_kernel_config('max_perc_undetected', perc_undetected, float, cfg)
 
     # Notify contact map instance of changes in scanning distance
     hic_genome.kernel_config = cfg
@@ -280,6 +273,8 @@ def cmd_quantify(args):
     # Only resize kernel matrix if explicitely requested
     km, kn = cfg["kernels"][0].shape
     if win_size != "auto":
+        if not win_size % 2:
+            raise ValueError("--win-size must be odd")
         for i, k in enumerate(cfg['kernels']):
             cfg['kernels'][i] = resize_kernel(k, factor=win_size / km)
         km = kn = win_size
@@ -300,7 +295,6 @@ def cmd_quantify(args):
             sub_pat = positions.loc[
                 (positions.chrom1 == mat.chr1) & (positions.chrom2 == mat.chr2)
             ]
-            sub_pat_idx = sub_pat.index.values
             # Convert genomic coordinates to bins for horizontal and vertical axes
             for ax in [1, 2]:
                 sub_pat_ax = sub_pat.loc[:, [f"chrom{ax}", f"pos{ax}"]].rename(
@@ -314,9 +308,11 @@ def cmd_quantify(args):
             if np.any(fall_out):
                 n_out = len(sub_pat_bins[fall_out])
                 sys.stderr.write(
-                    f"{n_out} entr{'ies' if n_out > 1 else 'y'} outside "
+                    f"\n{n_out} entr{'ies' if n_out > 1 else 'y'} outside "
                     "genomic coordinates of the Hi-C matrix will be ignored.\n"
                 )
+                sub_pat = sub_pat.loc[~fall_out, :]
+            sub_pat_idx = sub_pat.index.values
             # Convert bins from whole genome matrix to sub matrix
             sub_pat = hic_genome.get_sub_mat_pattern(
                 mat.chr1, mat.chr2, sub_pat
@@ -329,7 +325,7 @@ def cmd_quantify(args):
                 cfg,
                 kernel_matrix,
                 coords=np.array(sub_pat.loc[:, ["bin1", "bin2"]]),
-                full=full,
+                full=True,
                 tsvd=tsvd,
             )
 
@@ -384,6 +380,9 @@ def cmd_quantify(args):
                 "qvalue",
             ],
         ]
+        # Set p-values of invalid scores to nan
+        bed2d.loc[np.isnan(bed2d.score), 'pvalue'] = np.nan
+        bed2d.loc[np.isnan(bed2d.score), 'qvalue'] = np.nan
         cio.write_patterns(bed2d, f"{pattern}_quant", output)
         # bed2d.to_csv(
         #    output / f"{pattern}_quant.txt", sep="\t", header=True, index=False
@@ -398,6 +397,12 @@ def cmd_quantify(args):
                 pattern=cfg["name"], n=windows.shape[0]
             )
             windows_pileup = cid.pileup_patterns(windows)
+            # Symmetrize pileup for diagonal patterns
+            if not cfg['max_dist']:
+                # Replace nan below diag by 0
+                windows_pileup = np.nan_to_num(windows_pileup)
+                # Add transpose
+                windows_pileup += np.transpose(windows_pileup) - np.diag(np.diag(windows_pileup))
             pileup_plot(windows_pileup, name=pileup_fname, output=output)
 
 
@@ -420,6 +425,8 @@ def cmd_generate_config(args):
     # If a specific window size if requested, resize all kernels
     if win_size != "auto":
         win_size = int(win_size)
+        if not win_size % 2:
+            raise ValueError("--win-size must be odd")
         resize = lambda m: resize_kernel(m, factor=win_size / m.shape[0])
         cfg["kernels"] = [resize(k) for k in cfg["kernels"]]
     # Otherwise, just inherit window size from the kernel config
@@ -478,7 +485,7 @@ def _detect_sub_mat(data):
         config,
         kernel,
         dump=dump,
-        full=config["full"],
+        full=True,
         tsvd=config["tsvd"],
     )
     sub.contact_map.destroy_mat()
@@ -495,7 +502,6 @@ def cmd_detect(args):
     # Parse command line arguments for detect
     dump = args["--dump"]
     force_norm = args["--force-norm"]
-    full = args["--full"]
     interchrom = args["--inter"]
     iterations = args["--iterations"]
     kernel_config_path = args["--kernel-config"]
@@ -507,6 +513,7 @@ def cmd_detect(args):
     output = args["<output>"]
     pattern = args["--pattern"]
     pearson = args["--pearson"]
+    perc_zero = args["--perc-zero"]
     perc_undetected = args["--perc-undetected"]
     subsample = args["--subsample"]
     threads = int(args["--threads"])
@@ -552,6 +559,7 @@ def cmd_detect(args):
         "min_dist": (min_dist, int),
         "min_separation": (min_separation, int),
         "max_perc_undetected": (perc_undetected, float),
+        "max_perc_zero": (perc_zero, float),
     }
     cfg = cio.load_kernel_config(config_path, custom)
     for param_name, (param_value, param_type) in params.items():
@@ -559,7 +567,10 @@ def cmd_detect(args):
 
     # Resize kernels if requested
     if win_size != "auto":
-        resize = lambda m: resize_kernel(m, factor=int(win_size) / m.shape[0])
+        win_size = int(win_size)
+        if not win_size % 2:
+            raise ValueError("--win-size must be odd")
+        resize = lambda m: resize_kernel(m, factor=win_size / m.shape[0])
         cfg["kernels"] = [resize(k) for k in cfg["kernels"]]
 
     if interchrom:
@@ -588,12 +599,10 @@ def cmd_detect(args):
     all_windows = []
 
     ### 2: DETECTION ON EACH SUBMATRIX
-    pool = mp.Pool(threads)
     n_sub_mats = hic_genome.sub_mats.shape[0]
     # Loop over the different kernel matrices for input pattern
     run_id = 0
     # Use cfg to inform jobs whether they should run full convolution
-    cfg["full"] = full
     cfg["tsvd"] = tsvd
     total_runs = len(cfg["kernels"]) * cfg["max_iterations"]
     sys.stderr.write("Detecting patterns...\n")
@@ -616,7 +625,8 @@ def cmd_detect(args):
             sub_mat_results = []
             # Run in multiprocessing subprocesses
             if threads > 1:
-                dispatcher = pool.imap_unordered(
+                pool = mp.Pool(threads)
+                dispatcher = pool.imap(
                     _detect_sub_mat, sub_mat_data, 1
                 )
             else:
@@ -668,7 +678,9 @@ def cmd_detect(args):
     if len(all_coords) == 0:
         sys.stderr.write("No pattern detected ! Exiting.\n")
         sys.exit(0)
-
+    # Finish parallelized part
+    if threads > 1:
+        pool.close()
     # Combine patterns of all kernel matrices into a single array
     all_coords = pd.concat(all_coords, axis=0).reset_index(drop=True)
     # Combine all windows from different kernels into a single pile of windows
@@ -745,6 +757,12 @@ def cmd_detect(args):
             pattern=cfg["name"], n=all_windows.shape[0]
         )
         windows_pileup = cid.pileup_patterns(all_windows)
+        # Symmetrize pileup for diagonal patterns
+        if not cfg['max_dist']:
+            # Replace nan below diag by 0
+            windows_pileup = np.nan_to_num(windows_pileup)
+            # Add transpose
+            windows_pileup += np.transpose(windows_pileup) - np.diag(np.diag(windows_pileup))
         pileup_plot(windows_pileup, name=pileup_fname, output=output)
 
 
