@@ -128,13 +128,17 @@ class HicGenome:
         self.compute_max_dist()
         if sample is not None:
             sample = float(sample)
+            try:
+                _ = self.clr.info['sum']
+            except KeyError:
+                raise IOError("sum information missing from cool file. Please fix the file.")
             try:
                 if sample > self.clr.info["sum"]:
                     print(
                         "sample value is higher than total contacts,"
                         "skipping subsampling."
                     )
-                    self.sample = sample = None
+                    self.sample = None
                 elif sample > 1:
                     self.sample = sample / self.clr.info["sum"]
                 elif sample > 0:
@@ -193,12 +197,17 @@ class HicGenome:
                 cis_only=not self.inter,
                 store=True,
                 map=pool.imap_unordered,
+                ignore_diags=2,
+                max_iters=200,
+                min_nnz=10,
+                chunksize=10000000,
             )
+            pool.close()
             print("Whole genome matrix balanced")
             # Reload bins attribute to include the weight  column
             self.bins = self.clr.bins()[:]
         # Bins with NaN weight are missing, matrix already balanced
-        self.detectable_bins = np.where(np.isfinite(self.bins.weight))[0]
+        self.detectable_bins = np.flatnonzero(np.isfinite(self.bins.weight))
         print(
             f"Found {len(self.detectable_bins)} / {self.clr.shape[0]}"
             "detectable bins"
@@ -561,29 +570,31 @@ class ContactMap:
         self.matrix = self.clr.matrix(sparse=True, balance=False)[s1:e1, s2:e2]
         subsample = float(sub)
         if subsample < 0:
-            raise ValueError("Error: Subsample must be strictly positive.")
+            raise ValueError("Subsample must be strictly positive.")
         # If subsample is a proportion, convert it to contact count
-        elif subsample < 1:
+        elif subsample <= 1:
             subsample *= self.matrix.sum()
+        else:
+            raise ValueError("Subsample cannot be above 1")
         # If the contact count is lower than total, apply subsampling
+        subsample = int(subsample)
         if subsample < self.matrix.sum():
-            subsample = int(subsample)
             # print(f"Subsampling {subsample} contacts from {self.name}")
             # Apply subsampling on the raw contact matrix
             self.matrix = preproc.subsample_contacts(
                 self.matrix, int(subsample)
             )
             # Balance the subsampled submatrix using precomputed weights
-            if balance:
-                weights = self.clr.bins()['weight']  # view
-                bias_rows = weights[s1:e1].values
-                bias_cols = weights[s2:e2].values
-                self.matrix.data = (
-                    bias_rows[self.matrix.row] *
-                    bias_cols[self.matrix.col] *
-                    self.matrix.data
-                    )
-                self.matrix.data[np.isnan(self.matrix.data)] = 0
+        if balance:
+            weights = self.clr.bins()['weight']  # view
+            bias_rows = weights[s1:e1].values
+            bias_cols = weights[s2:e2].values
+            self.matrix.data = (
+                bias_rows[self.matrix.row] *
+                bias_cols[self.matrix.col] *
+                self.matrix.data
+                )
+            self.matrix.data[np.isnan(self.matrix.data)] = 0
 
     @DumpMatrix("01_process_inter")
     def preprocess_inter_matrix(self):
