@@ -16,8 +16,9 @@ def validate_patterns(
     conv_mat,
     detectable_bins,
     kernel_matrix,
-    max_undetected_perc,
     drop=True,
+    zero_tol=0.3,
+    missing_tol=0.75,
 ):
     """
     Given a list of pattern coordinates and a contact map, remove patterns in
@@ -39,8 +40,11 @@ def validate_patterns(
         rows and columns, respectively.
     kernel_matrix : numpy.array of float
         The kernel that was used for pattern detection on the Hi-C matrix.
-    max_undetected_perc : float
-        Proportion of undetectable pixels allowed in a pattern window to
+    zero_tol : float
+        Proportion of zero pixels allowed in a pattern window to
+        consider it valid.
+    missing_tol : float
+        Proportion of missing pixels allowed in a pattern window to
         consider it valid.
     drop : bool
         Whether to discard pattern coordinates and windows from patterns which
@@ -64,8 +68,8 @@ def validate_patterns(
     half_h, half_w = win_h // 2 + 1, win_w // 2 + 1
     # Store coords to drop
     blacklist = []
-    detectable_rows = set(detectable_bins[0])
-    detectable_cols = set(detectable_bins[1])
+    missing_rows = preproc.valid_to_missing(detectable_bins[0], matrix.shape[0])
+    missing_cols = preproc.valid_to_missing(detectable_bins[1], matrix.shape[1])
 
     # Copy coords object and append column for scores
     validated_coords = pd.DataFrame(
@@ -92,36 +96,35 @@ def validate_patterns(
             and left >= 0
             and right < matrix.shape[1]
         ):
-            # Get bin ids to use in window
-            win_rows, win_cols = (range(high, low), range(left, right))
             # Subset window from chrom matrix
             pattern_window = matrix[high:low, left:right].toarray()
-
             n_rows, n_cols = pattern_window.shape
-            tot_pixels = n_rows * n_cols
-            # Compute number of missing rows and cols
-            n_bad_rows = n_rows - len(
-                set(win_rows).intersection(detectable_rows)
-            )
-            n_bad_cols = n_cols - len(
-                set(win_cols).intersection(detectable_cols)
-            )
+            # Compute positions of missing bins relative to window
+            rel_miss_rows = missing_rows - high
+            rel_miss_rows = rel_miss_rows[
+                (rel_miss_rows >= 0) & (rel_miss_rows < n_rows)
+            ]
+            rel_miss_cols = missing_cols - left
+            rel_miss_cols = rel_miss_cols[
+                (rel_miss_cols >= 0) & (rel_miss_cols < n_cols)
+            ]
+            # Set missing bins to nan
+            pattern_window[rel_miss_rows, :] = np.nan
+            pattern_window[:, rel_miss_cols] = np.nan
 
-            # Number of undetected pixels is "bad rows area" + "bad cols
-            # area" - "bad rows x bad cols intersection"
-            tot_undetected_pixels = (
-                n_bad_rows * n_cols
-                + n_bad_cols * n_rows
-                - n_bad_rows * n_bad_cols
-            )
-            # Number of uncovered pixels
-            tot_zero_pixels = len(pattern_window[pattern_window == 0])
-            tot_missing_pixels = max(tot_undetected_pixels, tot_zero_pixels)
+            tot_pixels = n_rows * n_cols
+            # Number of uncovered or missing pixels
+            tot_zero_pixels = np.sum(pattern_window == 0)
+            tot_missing_pixels = np.sum(~np.isfinite(pattern_window))
             # The pattern should not contain more missing pixels that the max
             # value defined in kernel config. This includes both pixels from
             # undetectable bins and zero valued pixels in detectable bins.
-            if tot_missing_pixels / tot_pixels < max_undetected_perc / 100.0:
-                validated_coords.score[i] = conv_mat[l[0], l[1]]
+            prop_undetected = tot_missing_pixels / tot_pixels
+            prop_zero = tot_zero_pixels / (tot_pixels - tot_missing_pixels)
+            # Only compute scores for patterns with at least 25% detectable pixels
+            # (by default) and less than user-defined proportion of zero pixels
+            if (prop_undetected < missing_tol) and (prop_zero < zero_tol):
+                validated_coords.score[i] = conv_mat[p1, p2]
                 pattern_windows[i, :, :] = pattern_window
             else:
                 # Current pattern will be dropped due to undetectable bins
@@ -168,7 +171,7 @@ def pileup_patterns(pattern_windows):
         2D numpy array containing the pileup (arithmetic mean) of
         input windows.
     """
-    return np.apply_along_axis(np.nanmedian, 0, pattern_windows)
+    return np.apply_along_axis(np.nanmean, 0, pattern_windows)
 
 
 def pattern_detector(
@@ -252,6 +255,7 @@ def pattern_detector(
         missing_mask=missing_mask,
         tsvd=tsvd,
         pval=True,
+        missing_tol=kernel_config["max_perc_undetected"] / 100,
     )
     if dump:
         save_dump("03_normxcorr2", mat_conv)
@@ -268,6 +272,7 @@ def pattern_detector(
     # Only attempt detection if no input coordinates were given
     if run_mode == "detect":
         # Find foci of highly correlated pixels and pick local maxima
+        # coords, foci_mat = picker(np.abs(mat_log10_pvals), 5)
         coords, foci_mat = picker(mat_conv, kernel_config["pearson"],)
         # If nothing was detected, no point in resuming
         if coords is None:
@@ -278,6 +283,7 @@ def pattern_detector(
     det = [d.copy() for d in contact_map.detectable_bins]
     # Zero pad contact and convolution maps and shift missing bins and detected
     # pattern coords before validation if in full mode
+
     if full:
         mat = mat.tocoo()
         mat = preproc.zero_pad_sparse(mat, kh, kw, fmt="csr")
@@ -311,9 +317,10 @@ def pattern_detector(
         coords,
         mat,
         mat_conv.tocsr(),
-        contact_map.detectable_bins,
+        det,
         kernel_matrix,
-        kernel_config["max_perc_undetected"],
+        zero_tol=kernel_config["max_perc_zero"] / 100,
+        missing_tol=kernel_config["max_perc_undetected"] / 100,
         drop=True if run_mode == "detect" else False,
     )
 
@@ -359,10 +366,10 @@ def remove_neighbours(patterns, win_size=8):
     blacklist = set()
     for i, p in sorted_patterns.iterrows():
         if i not in blacklist:
-            close_patterns = np.where(
+            close_patterns = np.flatnonzero(
                 (np.abs(sorted_patterns.bin1 - p.bin1) < win_size)
                 & (np.abs(sorted_patterns.bin2 - p.bin2) < win_size)
-            )[0]
+            )
             close_patterns_idx = sorted_patterns.index.values[close_patterns]
             close_patterns_idx = close_patterns_idx[close_patterns_idx != i]
             for idx in close_patterns_idx:
@@ -420,7 +427,7 @@ def picker(mat_conv, pearson):
         # This is why we use focus_rank
         for focus_rank, focus_id in enumerate(np.unique(labelled_mat.data)):
             # Remember 1D indices of datapoint in focus
-            focus_idx = np.where(labelled_mat.data == focus_id)[0]
+            focus_idx = np.flatnonzero(labelled_mat.data == focus_id)
             focus_rows, focus_cols = (
                 labelled_mat.row[focus_idx],
                 labelled_mat.col[focus_idx],
@@ -477,7 +484,6 @@ def label_foci(matrix):
            [0 0 0 0]])
     """
     # step 1 : ensure that coo format is double ranked (row, col).
-    # TODO: we could find a trick to avoid this conversion step
     matrix_sp = sp.coo_matrix(sp.csr_matrix(matrix))
 
     nb_row, nb_col = matrix_sp.shape
@@ -503,7 +509,7 @@ def label_foci(matrix):
     diff_col_rc = col_rc[1:] - col_rc[:-1]
     # Right connected pixels are on the same row and neighboring cols
     right_connected_p = (diff_row_rc == 0) & (diff_col_rc == 1)
-    right_connected_k = np.where(right_connected_p)[0]
+    right_connected_k = np.flatnonzero(right_connected_p)
 
     right_node1 = right_connected_k
     right_node2 = right_connected_k + 1
@@ -517,7 +523,7 @@ def label_foci(matrix):
     diff_col_cr = col_cr[1:] - col_cr[:-1]
 
     lower_connected_p = (diff_row_cr == 1) & (diff_col_cr == 0)
-    lower_connected_k = np.where(lower_connected_p)[0]
+    lower_connected_k = np.flatnonzero(lower_connected_p)
 
     cr2rc = np.arange(len(coo_cr))[coo_cr]
     lower_node1 = cr2rc[lower_connected_k]
@@ -638,16 +644,10 @@ def _xcorr2_sparse(signal, kernel, threshold=1e-6):
         n_factors = kernel_l.shape[1]
         for f in range(n_factors):
             subkernel_l = sp.diags(
-                kernel_l[:, f],
-                np.arange(km),
-                shape=(sm - km + 1, sm),
-                format="dia",
+                kernel_l[:, f], np.arange(km), shape=(sm - km + 1, sm), format="dia",
             )
             subkernel_r = sp.diags(
-                kernel_r[f, :],
-                -np.arange(kn),
-                shape=(sn, sn - kn + 1),
-                format="dia",
+                kernel_r[f, :], -np.arange(kn), shape=(sn, sn - kn + 1), format="dia",
             )
             if f == 0:
                 out = (subkernel_l @ signal) @ subkernel_r
@@ -663,9 +663,7 @@ def _xcorr2_sparse(signal, kernel, threshold=1e-6):
             raise ValueError("cannot handle signal in dense format")
         # Check of kernel is constant (uniform)
         constant_kernel = np.nan
-        if np.allclose(
-            kernel, np.tile(kernel[0, 0], kernel.shape), rtol=1e-08
-        ):
+        if np.allclose(kernel, np.tile(kernel[0, 0], kernel.shape), rtol=1e-08):
             constant_kernel = kernel[0, 0]
 
         out = sp.csc_matrix((sm - km + 1, sn - kn + 1), dtype=np.float64)
@@ -679,20 +677,14 @@ def _xcorr2_sparse(signal, kernel, threshold=1e-6):
                 format="dia",
             )
             r_subkernel_sp = sp.diags(
-                np.ones(kn),
-                -np.arange(kn),
-                shape=(sn, sn - kn + 1),
-                format="dia",
+                np.ones(kn), -np.arange(kn), shape=(sn, sn - kn + 1), format="dia",
             )
             out = (l_subkernel_sp @ signal) @ r_subkernel_sp
         # Convolution code for general case
         else:
             for kj in range(kn):
                 subkernel_sp = sp.diags(
-                    kernel[:, kj],
-                    np.arange(km),
-                    shape=(sm - km + 1, sm),
-                    format="csr",
+                    kernel[:, kj], np.arange(km), shape=(sm - km + 1, sm), format="csr",
                 )
                 out += subkernel_sp.dot(signal[:, kj : sn - kn + 1 + kj])
 
@@ -734,16 +726,10 @@ def _xcorr2_dense(signal, kernel, threshold=1e-6):
         n_factors = kernel_l.shape[1]
         for f in range(n_factors):
             subkernel_l = sp.diags(
-                kernel_l[:, f],
-                np.arange(km),
-                shape=(sm - km + 1, sm),
-                format="dia",
+                kernel_l[:, f], np.arange(km), shape=(sm - km + 1, sm), format="dia",
             )
             subkernel_r = sp.diags(
-                kernel_r[f, :],
-                -np.arange(kn),
-                shape=(sn, sn - kn + 1),
-                format="dia",
+                kernel_r[f, :], -np.arange(kn), shape=(sn, sn - kn + 1), format="dia",
             )
             if f == 0:
                 out = (subkernel_l @ signal) @ subkernel_r
@@ -753,9 +739,7 @@ def _xcorr2_dense(signal, kernel, threshold=1e-6):
         km, kn = kernel.shape
         # Kernel (half) height and width
         constant_kernel = np.nan
-        if np.allclose(
-            kernel, np.tile(kernel[0, 0], kernel.shape), rtol=1e-08
-        ):
+        if np.allclose(kernel, np.tile(kernel[0, 0], kernel.shape), rtol=1e-08):
             constant_kernel = kernel[0, 0]
         out_wo_margin = np.zeros([sm - km + 1, sn - kn + 1])
         # Simplified convolution for the special case where kernel is constant:
@@ -767,24 +751,16 @@ def _xcorr2_dense(signal, kernel, threshold=1e-6):
                 format="dia",
             )
             r_subkernel_sp = sp.diags(
-                np.ones(kn),
-                -np.arange(kn),
-                shape=(sn, sn - kn + 1),
-                format="dia",
+                np.ones(kn), -np.arange(kn), shape=(sn, sn - kn + 1), format="dia",
             )
             out_wo_margin = (l_subkernel_sp @ signal) @ r_subkernel_sp
         # convolution code for general case
         else:
             for kj in range(kn):
                 subkernel_sp = sp.diags(
-                    kernel[:, kj],
-                    np.arange(km),
-                    shape=(sm - km + 1, sm),
-                    format="csr",
-                )
-                out_wo_margin += (
-                    subkernel_sp @ signal[:, kj : sn - kn + 1 + kj]
+                    kernel[:, kj], np.arange(km), shape=(sm - km + 1, sm), format="csr",
                 )
+                out_wo_margin += subkernel_sp @ signal[:, kj : sn - kn + 1 + kj]
 
     kh = (km - 1) // 2
     kw = (kn - 1) // 2
@@ -864,9 +840,7 @@ def normxcorr2(
         if not sp.issparse(missing_mask):
             raise ValueError("Missing mask must be a sparse matrix.")
         if not signal.shape == missing_mask.shape:
-            raise ValueError(
-                "Signal and missing mask do not have the same shape"
-            )
+            raise ValueError("Signal and missing mask do not have the same shape")
         if missing_mask.dtype != bool:
             raise ValueError(
                 f"Missing mask dtype is {missing_mask.dtype}. Should be bool."
@@ -875,8 +849,6 @@ def normxcorr2(
         if min(kernel.shape) >= max(signal.shape):
             raise ValueError("cannot have kernel bigger than signal")
         preproc.check_missing_mask(signal, missing_mask)
-        if missing_mask.nnz == 0:
-            missing_mask = None
 
     if sp.issparse(kernel):
         raise ValueError("cannot handle kernel in sparse format")
@@ -974,7 +946,6 @@ def _normxcorr2_sparse(
     # be centered on the edges.
     if full:
         # Create a vertical margin and use it to pad the signal
-        # TODO: delegate signal framing to frame
         tmp = sp.csr_matrix((mk - 1, ns))
         framed_sig = sp.vstack([tmp, signal, tmp], format=signal.format)
         # Same for the horizontal margin
@@ -984,10 +955,7 @@ def _normxcorr2_sparse(
         if missing_mask is not None:
             # Add margins around missing mask.
             framed_missing_mask = preproc.frame_missing_mask(
-                missing_mask,
-                kernel.shape,
-                sym_upper=sym_upper,
-                max_dist=max_dist,
+                missing_mask, kernel.shape, sym_upper=sym_upper, max_dist=max_dist,
             )
         else:
             framed_missing_mask = None
@@ -1004,9 +972,7 @@ def _normxcorr2_sparse(
         # out contains framed_sig mean
         out = xcorr2(framed_sig, kernel1 / kernel_size)
 
-        denom = xcorr2(framed_sig.power(2), kernel1 / kernel_size) - out.power(
-            2
-        )
+        denom = xcorr2(framed_sig.power(2), kernel1 / kernel_size) - out.power(2)
         denom = denom.sqrt() * kernel_std
         # quick and dirty hack to robustify: numerical-zeros are turned into
         # real-zeros
@@ -1015,10 +981,7 @@ def _normxcorr2_sparse(
 
         # store numerator directly in 'out' to avoid multiple replication of
         # data
-        out = (
-            xcorr2(framed_sig, kernel / kernel_size, tsvd=tsvd)
-            - out * kernel_mean
-        )
+        out = xcorr2(framed_sig, kernel / kernel_size, tsvd=tsvd) - out * kernel_mean
         # Multiply by invert since sparse division is not allowed
         out = out.multiply(denom)
 
@@ -1027,7 +990,6 @@ def _normxcorr2_sparse(
         preproc.check_missing_mask(framed_sig, framed_missing_mask)
         kernel_sum = np.sum(kernel)
         kernel_mean = kernel_sum / kernel_size
-        kernel_std = float(kernel.std())
         kernel2_sum = np.sum(kernel ** 2)
         kernel2_mean = kernel2_sum / kernel_size
         # Compute convolution of uniform kernel with missing mask to get number
@@ -1105,11 +1067,15 @@ def _normxcorr2_sparse(
         out = sp.triu(out)
     out = out.tocoo()
     out.data[~np.isfinite(out.data)] = 0.0
-    out.data[out.data < 0] = 0.0
+    # Remove near zero coeffs to spare memory
+    out.data[np.abs(out.data) < 0] = 0.0
+    # Prevent rounding errors to generate values above 1 or below -1
+    out.data[out.data < -1] = -1.0
+    out.data[out.data > 1] = 1.0
     out.eliminate_zeros()
     if pval:
         pvals = out.copy()
-        if full:
+        if full and framed_missing_mask is not None:
             try:
                 # Get number of values for each coeff
                 n_obs = ker1_coo.tocsr()[pvals.row, pvals.col].A1
@@ -1192,9 +1158,6 @@ def _normxcorr2_dense(
     if isinstance(kernel, np.matrix):
         kernel = np.array(kernel)
 
-    if missing_mask is not None:
-        return xcorr2(signal, kernel)
-
     kernel_size = mk * nk
     kernel1 = np.ones(kernel.shape)
     if full:
@@ -1203,31 +1166,22 @@ def _normxcorr2_dense(
         if missing_mask is not None:
             # Add margins around missing mask.
             framed_missing_mask = preproc.frame_missing_mask(
-                missing_mask,
-                kernel.shape,
-                sym_upper=sym_upper,
-                max_dist=max_dist,
+                missing_mask, kernel.shape, sym_upper=sym_upper, max_dist=max_dist,
             )
     else:
         framed_sig = signal
-        framed_missing_mask = None
         if missing_mask is not None:
             framed_missing_mask = missing_mask.copy()
     # Pearson correlation
     kernel_mean = float(kernel.mean())
     kernel_std = float(kernel.std())
 
-    kernel1 = np.ones(kernel.shape)
     framed_sig_mean = xcorr2(framed_sig, kernel1 / kernel_size)
 
     if missing_mask is None:
-        out = (
-            xcorr2(framed_sig, kernel / kernel_size)
-            - framed_sig_mean * kernel_mean
-        )
+        out = xcorr2(framed_sig, kernel / kernel_size) - framed_sig_mean * kernel_mean
         denom = kernel_std * np.sqrt(
-            xcorr2(framed_sig ** 2, kernel1 / kernel_size)
-            - framed_sig_mean ** 2
+            xcorr2(framed_sig ** 2, kernel1 / kernel_size) - framed_sig_mean ** 2
         )
         denom_0 = abs(denom) < 1e-10
         out[~denom_0] /= denom[~denom_0]
@@ -1267,7 +1221,10 @@ def _normxcorr2_dense(
     if sym_upper:
         out = np.triu(out)
     out[~np.isfinite(out)] = 0.0
-    out[out < 0] = 0.0
+    out[np.abs(out) < 0] = 0.0
+    # Prevent rounding errors to generate values above 1 or below -1
+    out[out < -1] = -1.0
+    out[out > 1] = 1.0
     if pval:
         if full:
             # Get number of values for each coeff
