@@ -56,12 +56,8 @@ def erase_missing(signal, valid_rows, valid_cols, sym_upper=True):
         erased = mask_mat.dot(signal).dot(mask_mat)
     else:
         # Get a boolean array of missing (1) and valid (0) rows
-        missing_rows = np.ones(signal.shape[0])
-        missing_cols = np.ones(signal.shape[1])
-        missing_rows[valid_rows] = 0
-        missing_cols[valid_cols] = 0
-        missing_rows = np.where(missing_rows)[0]
-        missing_cols = np.where(missing_cols)[0]
+        missing_rows = valid_to_missing(valid_rows, signal.shape[0])
+        missing_cols = valid_to_missing(valid_cols, signal.shape[1])
         erased = signal.copy()
         erased[missing_rows, :] = 0
         erased[:, missing_cols] = 0
@@ -88,7 +84,6 @@ def normalize(B, good_bins=None, iterations=10):
         The SCN normalised Hi-C matrix
     """
     if good_bins is None:
-        good_bins = np.arange(B.shape[0])
         r = B.copy()
     else:
         # Enforce removal of values in missing bins
@@ -286,7 +281,7 @@ def despeckle(matrix, th2=3):
 def get_detectable_bins(mat, n_mads=3, inter=False):
     """
     Returns lists of detectable indices after excluding low interacting bin
-    based on the distribution of pixel values in the matrix.
+    based on the proportion of zero pixel values in the matrix bins.
 
     Parameters
     ----------
@@ -307,10 +302,11 @@ def get_detectable_bins(mat, n_mads=3, inter=False):
     -------
     """
     matrix = mat.copy()
+    matrix.eliminate_zeros()
     mad = lambda x: ss.median_absolute_deviation(x, nan_policy="omit")
     if not inter:
         if matrix.shape[0] != matrix.shape[1]:
-            raise ValueError("intrachromosomal matrices must be symmetric.")
+            raise ValueError("Intrachromosomal matrices must be symmetric.")
         # Replace nonzero pixels by ones to work on prop. of nonzero pixels
         matrix.data = np.ones(matrix.data.shape)
         # Compute number of nonzero values in each bin
@@ -322,7 +318,7 @@ def get_detectable_bins(mat, n_mads=3, inter=False):
         detect_threshold = max(1, sum_med - sum_mad * n_mads)
 
         # Removal of poor interacting rows and columns
-        good_bins = np.where(sum_bins > detect_threshold)[0]
+        good_bins = np.flatnonzero(sum_bins >= detect_threshold)
         good_bins = (good_bins, good_bins)
     else:
         # Adapted for asymetric matrices (need to compute rows and columns)
@@ -331,8 +327,8 @@ def get_detectable_bins(mat, n_mads=3, inter=False):
         med_rows, med_cols = np.median(sum_rows), np.median(sum_cols)
         detect_threshold_rows = max(1, med_rows - mad_rows * n_mads)
         detect_threshold_cols = max(1, med_cols - mad_cols * n_mads)
-        good_rows = np.where(sum_rows > detect_threshold_rows)[0]
-        good_cols = np.where(sum_cols > detect_threshold_cols)[0]
+        good_rows = np.flatnonzero(sum_rows > detect_threshold_rows)
+        good_cols = np.flatnonzero(sum_cols > detect_threshold_cols)
         good_bins = (good_rows, good_cols)
     return good_bins
 
@@ -342,7 +338,7 @@ def detrend(
     detectable_bins=None,
     max_dist=None,
     smooth=False,
-    fun=np.nanmedian,
+    fun=np.nanmean,
     max_val=10,
 ):
     """
@@ -447,7 +443,7 @@ def signal_to_noise_threshold(matrix, detectable_bins):
     threshold_noise = 1.0
     snr[np.isnan(snr)] = 0.0
     try:
-        max_dist = min(np.where(snr < threshold_noise)[0])
+        max_dist = min(np.flatnonzero(snr < threshold_noise))
     except ValueError:
         max_dist = matrix.shape[0]
     return max_dist
@@ -602,10 +598,10 @@ def frame_missing_mask(mask, kernel_shape, sym_upper=False, max_dist=None):
         framed_mask = framed_mask.tolil()
         # Add margin below diagonal
         big_k = max(nk, mk)
-        dia_margin = np.ones(big_k // 2 - 1)
-        dia_offsets = np.arange(-1, -big_k // 2 + 1, -1)
+        dia_margins = np.ones(big_k)
+        dia_offsets = np.arange(-1, -big_k-1, -1)
         framed_mask += sp.diags(
-            dia_margin,
+            dia_margins,
             dia_offsets,
             shape=framed_mask.shape,
             format="lil",
@@ -686,17 +682,12 @@ def make_missing_mask(
         raise ValueError("Rectangular matrices cannot be upper symmetric")
 
     # Get a boolean array of missing (1) and valid (0) rows
-    missing_rows = np.ones(sm, dtype=bool)
-    missing_rows[valid_rows] = False
-    missing_rows = np.where(missing_rows)[0]
+    missing_rows = valid_to_missing(valid_rows, sm)
     # When matrix is sym., rows and cols are synonym, no need to compute 2x
     if sym_upper:
         missing_cols = missing_rows
     else:
-        missing_cols = np.ones(sn, dtype=bool)
-        missing_cols[valid_cols] = False
-        missing_cols = np.where(missing_cols)[0]
-
+        missing_cols = valid_to_missing(valid_cols, sn)
     # If upper sym., fill only upper diag up to max_dist.
     # E. g. with bins 1 and 3 missing
     # and a max_dist of 1:
@@ -889,9 +880,9 @@ def resize_kernel(
     """
     km, kn = kernel.shape
     if km != kn:
-        ValueError("kernel must be square.")
+        raise ValueError("kernel must be square.")
     if not (km % 2) or not (kn % 2):
-        ValueError("kernel size must be odd.")
+        raise ValueError("kernel size must be odd.")
 
     if factor is not None:
         if kernel_res is not None or signal_res is not None:
@@ -948,7 +939,7 @@ def factorise_kernel(kernel, prop_info=0.999):
     u, sigma, v = la.svd(kernel)
     total_info = np.sum(sigma ** 2)
     # Compute min. number of singular vectors to retain enough info
-    keep_k = np.where(np.cumsum(sigma ** 2) > prop_info * total_info)[0][0] + 1
+    keep_k = np.flatnonzero(np.cumsum(sigma ** 2) > prop_info * total_info)[0] + 1
     if keep_k > np.floor(min(kernel.shape) / 2):
         sys.stderr.write(
             f"Warning: Kernel factorisation required {keep_k} singular,"
@@ -963,3 +954,33 @@ def factorise_kernel(kernel, prop_info=0.999):
         v[i, :] *= np.sqrt(sigma[i])
 
     return (u, v)
+
+
+def valid_to_missing(valid, size):
+    """
+    Given an array of valid indices, return the corrsesponding array of missing
+    indices.
+
+    Parameters
+    ---------
+    valid : numpy.ndarray of ints
+        The valid indices.
+    size : int
+        The size of the matrix (maximum possible index + 1).
+
+    Return
+    ------
+    missing : numpy.ndarray of ints
+        The missing indices.
+
+    """
+    missing = np.ones(size, dtype=bool)
+    try:
+        missing[valid] = False
+    # In case there is no valid index
+    except IndexError:
+        pass
+    missing = np.flatnonzero(missing)
+    return missing
+
+
